@@ -21,7 +21,7 @@ import java.nio.ByteBuffer;
 /** File
  *
  * @author Mladen Turk
- * @version $Revision: 466585 $, $Date: 2006-10-21 16:16:34 -0600 (Sat, 21 Oct 2006) $
+ * @version $Revision: 730523 $, $Date: 2009-01-01 03:52:46 -0700 (Thu, 01 Jan 2009) $
  */
 
 public class File {
@@ -692,6 +692,16 @@ public class File {
      */
     public static native int stat(FileInfo finfo, String fname, int wanted, long pool);
 
+    /**
+     * Get the specified file's stats.  The file is specified by filename,
+     * instead of using a pre-opened file.
+     * @param fname The name of the file to stat.
+     * @param wanted The desired apr_finfo_t fields, as a bit flag of APR_FINFO_ values
+     * @param pool the pool to use to allocate the new file.
+     * @return FileInfo object.
+     */
+    public static native FileInfo getStat(String fname, int wanted, long pool);
+
     /**
      * Get the specified file's stats.
      * @param finfo Where to store the information about the file.
@@ -700,4 +710,13 @@ public class File {
      */
     public static native int infoGet(FileInfo finfo, int wanted, long thefile);
 
+
+    /**
+     * Get the specified file's stats.
+     * @param wanted The desired apr_finfo_t fields, as a bit flag of APR_FINFO_ values
+     * @param thefile The file to get information about.
+     * @return FileInfo object.
+     */
+    public static native FileInfo getInfo(int wanted, long thefile);
+
 }
