@@ -20,7 +20,7 @@ package org.apache.tomcat.jni;
 /** SSL Context
  *
  * @author Mladen Turk
- * @version $Revision: 466585 $, $Date: 2006-10-21 16:16:34 -0600 (Sat, 21 Oct 2006) $
+ * @version $Revision: 730528 $, $Date: 2009-01-01 04:24:40 -0700 (Thu, 01 Jan 2009) $
  */
 
 public final class SSLContext {
@@ -225,6 +225,13 @@ public final class SSLContext {
                                                   String path)
         throws Exception;
 
+    /**
+     * Set file for randomness
+     * @param ctx Server or Client context to use.
+     * @param file random file.
+     */
+    public static native void setRandom(long ctx, String file);
+
     /**
      * Set SSL connection shutdown type
      * <br />
@@ -237,7 +244,7 @@ public final class SSLContext {
      * @param ctx Server or Client context to use.
      * @param type Shutdown type to use.
      */
-    public static native void setShutdowType(long ctx, int type);
+    public static native void setShutdownType(long ctx, int type);
 
     /**
      * Set Type of Client Certificate verification and Maximum depth of CA Certificates
