@@ -17,7 +17,7 @@
 /*
  *
  * @author Mladen Turk
- * @version $Revision: 687661 $, $Date: 2008-08-21 04:20:21 -0500 (Thu, 21 Aug 2008) $
+ * @version $Revision: 730523 $, $Date: 2009-01-01 03:52:46 -0700 (Thu, 01 Jan 2009) $
  */
 
 #include "tcn.h"
@@ -53,6 +53,10 @@ TCN_IMPLEMENT_CALL(jlong, Address, info)(TCN_STDARGS,
                 break; /* Done */
             sl = sl->next;
         }
+        /* If we don't find an IPv6 address, use the original one */
+        if (sl == NULL) {
+            sl = sa;
+        }
     }
 #endif
 
@@ -99,8 +103,8 @@ cleanup:
     return P2J(sa);
 }
 
-TCN_IMPLEMENT_CALL(jint, Address, equal)(TCN_STDARGS,
-                                         jlong a, jlong b)
+TCN_IMPLEMENT_CALL(jboolean, Address, equal)(TCN_STDARGS,
+                                             jlong a, jlong b)
 {
     apr_sockaddr_t *sa = J2P(a, apr_sockaddr_t *);
     apr_sockaddr_t *sb = J2P(b, apr_sockaddr_t *);
