@@ -17,7 +17,7 @@
 /*
  *
  * @author Mladen Turk
- * @version $Revision: 661428 $, $Date: 2008-05-29 12:08:20 -0600 (Thu, 29 May 2008) $
+ * @version $Revision: 693044 $, $Date: 2008-09-08 04:19:13 -0600 (Mon, 08 Sep 2008) $
  */
 
 #include "tcn.h"
@@ -281,7 +281,7 @@ TCN_IMPLEMENT_CALL(jint, Poll, poll)(TCN_STDARGS, jlong pollset,
     tcn_pollset_t *p = J2P(pollset,  tcn_pollset_t *);
     apr_int32_t  i, num = 0;
     apr_status_t rv = APR_SUCCESS;
-    apr_time_t now;
+    apr_time_t now = 0;
     apr_interval_time_t ptime = J2T(timeout);
     UNREFERENCED(o);
     TCN_ASSERT(pollset != 0);
