@@ -23,7 +23,7 @@ package org.apache.catalina;
  * Global constants that are applicable to multiple packages within Catalina.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 466608 $ $Date: 2006-10-21 17:10:15 -0600 (Sat, 21 Oct 2006) $
+ * @version $Revision: 757764 $ $Date: 2009-03-24 06:43:25 -0600 (Tue, 24 Mar 2009) $
  */
 
 public final class Globals {
@@ -269,14 +269,18 @@ public final class Globals {
      * The name of the cookie used to pass the session identifier back
      * and forth with the client.
      */
-    public static final String SESSION_COOKIE_NAME = "JSESSIONID";
+    public static final String SESSION_COOKIE_NAME =
+        System.getProperty("org.apache.catalina.SESSION_COOKIE_NAME",
+                "JSESSIONID");
 
 
     /**
      * The name of the path parameter used to pass the session identifier
      * back and forth with the client.
      */
-    public static final String SESSION_PARAMETER_NAME = "jsessionid";
+    public static final String SESSION_PARAMETER_NAME =
+        System.getProperty("org.apache.catalina.SESSION_PARAMETER_NAME",
+                "jsessionid");
 
 
     /**
