@@ -44,7 +44,7 @@ import org.apache.catalina.security.SecurityUtil;
  * @author Craig R. McClanahan
  * @author Remy Maucherat
  * @author Jean-Francois Arcand
- * @version $Revision: 466608 $ $Date: 2006-10-21 17:10:15 -0600 (Sat, 21 Oct 2006) $
+ * @version $Revision: 757782 $ $Date: 2009-03-24 07:18:15 -0600 (Tue, 24 Mar 2009) $
  */
 
 public class RequestFacade implements HttpServletRequest {
@@ -931,4 +931,7 @@ public class RequestFacade implements HttpServletRequest {
         return request.getRemotePort();
     }
 
+    public boolean getAllowTrace() {
+        return request.getConnector().getAllowTrace();
+    }
 }
