@@ -102,7 +102,7 @@ import org.apache.tomcat.util.IntrospectionUtils;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 689624 $ $Date: 2008-08-27 16:26:05 -0500 (Wed, 27 Aug 2008) $
+ * @version $Revision: 781592 $ $Date: 2009-06-03 15:37:13 -0600 (Wed, 03 Jun 2009) $
  */
 public class WebappClassLoader
     extends URLClassLoader
@@ -1826,7 +1826,11 @@ public class WebappClassLoader
             return clazz;
 
         synchronized (this) {
-            if (entry.binaryContent == null && entry.loadedClass == null)
+            clazz = entry.loadedClass;
+            if (clazz != null)
+                return clazz;
+
+            if (entry.binaryContent == null)
                 throw new ClassNotFoundException(name);
 
             // Looking up the package
@@ -1872,19 +1876,22 @@ public class WebappClassLoader
     
             }
 
-            if (entry.loadedClass == null) {
+            try {
                 clazz = defineClass(name, entry.binaryContent, 0,
                         entry.binaryContent.length, 
                         new CodeSource(entry.codeBase, entry.certificates));
-                entry.loadedClass = clazz;
-                entry.binaryContent = null;
-                entry.source = null;
-                entry.codeBase = null;
-                entry.manifest = null;
-                entry.certificates = null;
-            } else {
-                clazz = entry.loadedClass;
+            } catch (UnsupportedClassVersionError ucve) {
+                throw new UnsupportedClassVersionError(
+                        ucve.getLocalizedMessage() + " " +
+                        sm.getString("webappClassLoader.wrongVersion",
+                                name));
             }
+            entry.loadedClass = clazz;
+            entry.binaryContent = null;
+            entry.source = null;
+            entry.codeBase = null;
+            entry.manifest = null;
+            entry.certificates = null;
         }
         
         return clazz;
