@@ -47,7 +47,7 @@ import org.apache.catalina.util.StringManager;
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
-* @version $Revision: 685835 $ $Date: 2008-08-14 02:37:23 -0700 (Thu, 14 Aug 2008) $
+* @version $Revision: 781379 $ $Date: 2009-06-03 07:58:22 -0600 (Wed, 03 Jun 2009) $
 */
 
 public class JDBCRealm
@@ -393,9 +393,10 @@ public class JDBCRealm
                                                String username,
                                                String credentials) {
 
-        // No user - can't possibly authenticate
-        if (username == null) {
-            return (null);
+        // No user or no credentials
+        // Can't possibly authenticate, don't bother the database then
+        if (username == null || credentials == null) {
+            return null;
         }
 
         // Look up the user's credentials
