@@ -61,7 +61,7 @@ import org.apache.commons.modeler.Registry;
  * location) are identical to those currently supported by Tomcat 3.X.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 681143 $ $Date: 2008-07-30 11:49:50 -0600 (Wed, 30 Jul 2008) $
+ * @version $Revision: 757737 $ $Date: 2009-03-24 05:25:50 -0600 (Tue, 24 Mar 2009) $
  */
 
 public abstract class RealmBase
@@ -785,9 +785,12 @@ public abstract class RealmBase
                     log.debug("  No user authenticated, cannot grant access");
             } else {
                 for (int j = 0; j < roles.length; j++) {
-                    if (hasRole(principal, roles[j]))
+                    if (hasRole(principal, roles[j])) {
                         status = true;
-                    if( log.isDebugEnabled() )
+                        if( log.isDebugEnabled() )
+                            log.debug( "Role found:  " + roles[j]);
+                    }
+                    else if( log.isDebugEnabled() )
                         log.debug( "No role found:  " + roles[j]);
                 }
             }
