@@ -75,7 +75,7 @@ import org.apache.catalina.security.SecurityUtil;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
- * @version $Revision: 685826 $ $Date: 2008-08-14 02:11:55 -0700 (Thu, 14 Aug 2008) $
+ * @version $Revision: 795041 $ $Date: 2009-07-17 05:16:05 -0600 (Fri, 17 Jul 2009) $
  */
 
 public class StandardSession
@@ -167,7 +167,7 @@ public class StandardSession
      * certain IllegalStateException tests.  NOTE:  This value is not
      * included in the serialized version of this object.
      */
-    protected transient boolean expiring = false;
+    protected transient volatile boolean expiring = false;
 
 
     /**
@@ -192,7 +192,7 @@ public class StandardSession
     /**
      * The last accessed time for this Session.
      */
-    protected long lastAccessedTime = creationTime;
+    protected volatile long lastAccessedTime = creationTime;
 
 
     /**
@@ -224,7 +224,7 @@ public class StandardSession
     /**
      * Flag indicating whether this session is valid or not.
      */
-    protected boolean isValid = false;
+    protected volatile boolean isValid = false;
 
     
     /**
@@ -267,13 +267,13 @@ public class StandardSession
     /**
      * The current accessed time for this session.
      */
-    protected long thisAccessedTime = creationTime;
+    protected volatile long thisAccessedTime = creationTime;
 
 
     /**
      * The access count for this session.
      */
-    protected transient int accessCount = 0;
+    protected volatile transient int accessCount = 0;
 
     private Object lock = new Object();
 
@@ -666,15 +666,20 @@ public class StandardSession
      */
     public void expire(boolean notify) {
 
-        // Mark this session as "being expired" if needed
-        if (expiring)
+        // Check to see if expire is in progress or has previously been called
+        if (expiring || !isValid)
             return;
 
         synchronized (this) {
+            // Check again, now we are inside the sync so this code only runs once
+            // Double check locking - expiring and isValid need to be volatile
+            if (expiring || !isValid)
+                return;
 
             if (manager == null)
                 return;
 
+            // Mark this session as "being expired"
             expiring = true;
         
             // Notify interested application event listeners
@@ -711,7 +716,9 @@ public class StandardSession
                     }
                 }
             }
-            accessCount = 0;
+            synchronized(lock) {
+                accessCount = 0;
+            }
             setValid(false);
 
             /*
@@ -851,7 +858,9 @@ public class StandardSession
         id = null;
         lastAccessedTime = 0L;
         maxInactiveInterval = -1;
-        accessCount = 0;
+        synchronized(lock) {
+            accessCount = 0;
+        }
         notes.clear();
         setPrincipal(null);
         isNew = false;
