@@ -64,7 +64,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 685757 $ $Date: 2008-08-13 21:10:28 -0600 (Wed, 13 Aug 2008) $
+ * @version $Revision: 781723 $ $Date: 2009-06-04 06:48:13 -0600 (Thu, 04 Jun 2009) $
  */
 
 public class ContextConfig
@@ -889,7 +889,7 @@ public class ContextConfig
             }
         }
 
-        if (docBase.startsWith(canonicalAppBase.getPath())) {
+        if (docBase.startsWith(canonicalAppBase.getPath() + File.separatorChar)) {
             docBase = docBase.substring(canonicalAppBase.getPath().length());
             docBase = docBase.replace(File.separatorChar, '/');
             if (docBase.startsWith("/")) {
