@@ -52,7 +52,7 @@ import org.apache.tomcat.util.digester.Digester;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 685757 $ $Date: 2008-08-13 21:10:28 -0600 (Wed, 13 Aug 2008) $
+ * @version $Revision: 784613 $ $Date: 2009-06-14 14:11:44 -0600 (Sun, 14 Jun 2009) $
  */
 public class HostConfig
     implements LifecycleListener {
@@ -251,7 +251,7 @@ public class HostConfig
     }
 
     /**
-     * Get the server.xml <host> attribute's xmlValidation.
+     * Get the server.xml &lt;host&gt; attribute's xmlValidation.
      * @return true if validation is enabled.
      *
      */
@@ -260,7 +260,7 @@ public class HostConfig
     }
 
     /**
-     * Get the server.xml <host> attribute's xmlNamespaceAware.
+     * Get the server.xml &lt;host&gt; attribute's xmlNamespaceAware.
      * @return true if namespace awarenes is enabled.
      *
      */
@@ -1020,7 +1020,7 @@ public class HostConfig
                         try {
                             File current = new File(resources[j]);
                             current = current.getCanonicalFile();
-                            if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath()))
+                            if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath() + File.separator))
                                     || (current.getAbsolutePath().startsWith(configBase().getAbsolutePath()))) {
                                 if (log.isDebugEnabled())
                                     log.debug("Delete " + current);
@@ -1060,7 +1060,7 @@ public class HostConfig
                     try {
                         File current = new File(resources[j]);
                         current = current.getCanonicalFile();
-                        if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath()))
+                        if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath() + File.separator))
                             || (current.getAbsolutePath().startsWith(configBase().getAbsolutePath()))) {
                             if (log.isDebugEnabled())
                                 log.debug("Delete " + current);
@@ -1077,7 +1077,7 @@ public class HostConfig
                     try {
                         File current = new File(resources2[j]);
                         current = current.getCanonicalFile();
-                        if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath()))
+                        if ((current.getAbsolutePath().startsWith(appBase().getAbsolutePath() + File.separator))
                             || ((current.getAbsolutePath().startsWith(configBase().getAbsolutePath())
                                  && (current.getAbsolutePath().endsWith(".xml"))))) {
                             if (log.isDebugEnabled())
