@@ -68,7 +68,7 @@ import org.apache.catalina.core.StandardContext;
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
  * @author Peter Rossbach
- * @version $Revision: 571202 $ $Date: 2007-08-30 09:14:48 -0600 (Thu, 30 Aug 2007) $
+ * @version $Revision: 790348 $ $Date: 2009-07-01 14:01:20 -0600 (Wed, 01 Jul 2009) $
  */
 
 public class DeltaManager extends ManagerBase implements Lifecycle,
@@ -1046,7 +1046,7 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
                 cluster.send(msg, mbr);
                 if (log.isWarnEnabled())
                     log.warn(sm.getString("deltaManager.waitForSessionState",
-                            getName(), mbr));
+                            getName(), mbr, new Integer(getStateTransferTimeout())));
                 // FIXME At sender ack mode this method check only the state transfer and resend is a problem!
                 waitForSendAllSessions(beforeSendTime);
             } finally {
@@ -1664,10 +1664,12 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
         session.setMaxInactiveInterval(getMaxInactiveInterval());
         session.setExpireTolerance(this.expireTolerance);
         session.access();
-        if(notifySessionListenersOnReplication)
+        if(notifySessionListenersOnReplication) {
             session.setId(msg.getSessionID());
-        else
+        } else {
             session.setIdInternal(msg.getSessionID());
+            add(session);
+        }
         session.resetDeltaRequest();
         session.endAccess();
 
