@@ -42,7 +42,7 @@ import org.apache.catalina.cluster.util.IQueue;
  * </ul>
  * FIXME: refactor code duplications with AsyncSocketSender => configurable or extract super class 
  * @author Peter Rossbach ( idea comes form Rainer Jung)
- * @version $Revision: 565198 $ $Date: 2007-08-12 17:22:23 -0600 (Sun, 12 Aug 2007) $
+ * @version $Revision: 790304 $ $Date: 2009-07-01 12:47:07 -0600 (Wed, 01 Jul 2009) $
  * @since 5.5.9
  */
 public class FastAsyncSocketSender extends DataSender {
@@ -91,6 +91,13 @@ public class FastAsyncSocketSender extends DataSender {
 
     private int threadPriority = Thread.NORM_PRIORITY;;
 
+    /**
+     * Separate mutex for our local state.
+     * We keep the synchronization independent of the synchronization
+     * in the super class DataSender.
+     */
+    private final Object mutex = new Object();
+
     // ------------------------------------------------------------- Constructor
 
     /**
@@ -353,7 +360,7 @@ public class FastAsyncSocketSender extends DataSender {
     public void sendMessage(ClusterData data)
             throws java.io.IOException {
         queue.add(data.getUniqueId(), data);
-        synchronized (this) {
+        synchronized (mutex) {
             inQueueCounter++;
             if(queueThread != null)
                 queueThread.incQueuedNrOfBytes(data.getMessage().length);
@@ -455,15 +462,15 @@ public class FastAsyncSocketSender extends DataSender {
             return queuedNrOfBytes;
         }
         
-        protected synchronized void setQueuedNrOfBytes(long queuedNrOfBytes) {
+        protected void setQueuedNrOfBytes(long queuedNrOfBytes) {
             this.queuedNrOfBytes = queuedNrOfBytes;
         }
 
-        protected synchronized void incQueuedNrOfBytes(long size) {
+        protected void incQueuedNrOfBytes(long size) {
             queuedNrOfBytes += size;
         }
         
-        protected synchronized void decQueuedNrOfBytes(long size) {
+        protected void decQueuedNrOfBytes(long size) {
             queuedNrOfBytes -= size;
         }
 
@@ -561,8 +568,10 @@ public class FastAsyncSocketSender extends DataSender {
                                 .getKey()), x);
                     }
                } finally {
-                    outQueueCounter++;
-                    decQueuedNrOfBytes(messagesize);
+                    synchronized (sender.mutex) {
+                        outQueueCounter++;
+                        decQueuedNrOfBytes(messagesize);
+                    }
                 }
                 entry = entry.next();
             } while (keepRunning && entry != null);
