@@ -577,6 +577,9 @@ class JspDocumentParser
                         lastCh = ch;
                     }
                 } else if (lastCh == '\\' && ch == '$') {
+                    if (pageInfo.isELIgnored()) {
+                        ttext.write('\\');
+                    }
                     ttext.write('$');
                     ch = 0;  // Not start of EL anymore
                 } else {
